/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc;

import java.util.*;

/**
 * The CTCP interpretor.
 */
public class CTCPInterpretor extends BasicInterpretor
{
  /**
   * The CTCPFilter.
   */
  protected CTCPFilter _filter;

  /**
   * Create a new CTCPInterpretor.
   * @param config global configuration.
   * @param next next interpretor to use if command is unknown.
   * @param filter CTCPFilter to use.
   */
  public CTCPInterpretor(IRCConfiguration config,Interpretor next,CTCPFilter filter)
  {
    super(config,next);
    _filter=filter;
  }

  protected void handleCommand(Source source,String cmd,String[] parts,String[] cumul)
  {
    try
    {
      if(cmd.equals("ctcp"))
      {
        test(cmd,parts,1);
        if(parts[1].toLowerCase().equals("ping"))
        {
          test(cmd,parts,2);
          _filter.ping(source.getServer(),parts[2]);
        }
        else if(parts[1].toLowerCase().equals("action"))
        {
          test(cmd,parts,2);
          if(source.talkable())
          {
            _filter.action(source.getServer(),source.getName(),cumul[2]);
            source.action(source.getServer().getNick(),cumul[2]);
          }
          else
          {
            source.report(getText(TextProvider.INTERPRETOR_NOT_ON_CHANNEL));
          }
        }
        else if(parts[1].toLowerCase().equals("dcc"))
        {
          test(cmd,parts,2);
          if(parts[2].toLowerCase().equals("chat"))
          {
            test(cmd,parts,3);
            _filter.chat(source.getServer(),parts[3]);
          }
          else if(parts[2].toLowerCase().equals("send"))
          {
            test(cmd,parts,4);
            String file=cumul[4];
            _filter.sendFile(source.getServer(),parts[3],file);
          }
          else
          {
            source.report(getText(TextProvider.INTERPRETOR_UNKNOWN_DCC,parts[2]));
          }
        }
        else if(parts[1].toLowerCase().equals("raw"))
        {
          test(cmd,parts,3);
          _filter.genericSend(source.getServer(),parts[2],cumul[3]);
        }
        else
        {
          test(cmd,parts,2);
          _filter.genericSend(source.getServer(),parts[2],parts[1]);
        }
      }
      else
      {
        super.handleCommand(source,cmd,parts,cumul);
      }
    }
    catch(NotEnoughParametersException ex)
    {
      source.report(getText(TextProvider.INTERPRETOR_INSUFFICIENT_PARAMETERS,ex.getMessage()));
    }
  }
}

