/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc.gui;

import irc.*;
import irc.style.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import irc.tree.*;
import irc.gui.pixx.*;

class ChanListNameComparator implements irc.tree.Comparator
{
  public int compare(Object o1,Object o2)
  {
    ChannelInfo info1=(ChannelInfo)o1;
    ChannelInfo info2=(ChannelInfo)o2;
    return info2.userCount-info1.userCount;
  }
}

/**
 * The AWT channel list.
 */
public class AWTChanList extends AWTSource implements ChanListListener
{
  protected PixxHorizontalScrollBar _hscroll;
  private SortedList _sortedList;

  /**
   * Create a new AWTChanList.
   * @param config the global irc configuration.
   * @param list the source channel list.
   */
  public AWTChanList(IRCConfiguration config,ChanList list)
  {
    super(config,list,true);

    _ircConfiguration=config;
    _sortedList=new SortedList(new ChanListNameComparator());
    list.addChanListListener(this);

    _hscroll=new PixxHorizontalScrollBar(_ircConfiguration,0,0,0.1);
    _hscroll.addPixxScrollBarListener(this);

    _list.setWrap(false);

    remove(_textField);
    add(_hscroll,"South");

    setTitle(getText(TextProvider.SOURCE_CHANLIST,getChanList().getName()));
    _source.activate();
  }

  public void release()
  {
    ((ChanList)_source).removeChanListListeners(this);
    _hscroll.removePixxScrollBarListener(this);
    _hscroll.release();
    _hscroll=null;
    super.release();
  }

  public String getShortTitle()
  {
    return getText(TextProvider.GUI_CHANNELS);
  }

  public void setFieldText(String txt)
  {
  }

  public String getFieldText()
  {
    return "";
  }

  public void validateText()
  {
  }

  /**
   * Get the source chanlist.
   * @return source chanlist.
   */
  public ChanList getChanList()
  {
    return (ChanList)getSource();
  }

  public void channelBegin()
  {
    clear();
    print(_ircConfiguration.getText(TextProvider.SOURCE_CHANLIST_RETREIVING));
    _list.setFirst(0);
    _sortedList=new SortedList(new ChanListNameComparator());
  }

  public void channelEnd()
  {
    clear();
    String[] lines=new String[_sortedList.getSize()];
    Enumeration e=_sortedList.getItems();
    int i=0;
    while(e.hasMoreElements())
    {
      ChannelInfo ci=(ChannelInfo)e.nextElement();
      lines[i++]=format(ci);
    }
    _list.addLines(lines);
    _list.setFirst(0);
    _scroll.setMaximum(_list.getLineCount()-1);
    _scroll.setValue(_list.getLast());
    _hscroll.setMaximum(_list.getLogicalWidth()/10);
  }

  private String format(ChannelInfo item)
  {
    String msg=item.name;
    String count=""+item.userCount;
    for(int i=0;i<20-item.name.length();i++) msg+=" ";
    msg+="   "+item.userCount;
    for(int i=0;i<5-count.length();i++) msg+=" ";
    msg+="   "+item.topic;
    return msg;
  }

  public void channelAdded(ChannelInfo item)
  {
    _sortedList.add(item);
  }

  public void valueChanged(PixxScrollBar pixScrollBar)
  {
    if(pixScrollBar==_hscroll)
      _list.setLeft(_hscroll.getValue()*10);
    else super.valueChanged(pixScrollBar);
  }

  public void virtualSizeChanged(StyledList lis)
  {
    _hscroll.setMaximum(_list.getLogicalWidth()/10);
    super.virtualSizeChanged(lis);
  }
}
