/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc.gui;

import java.util.*;
import irc.*;
import irc.style.*;
import java.awt.*;
import java.awt.event.*;
import irc.gui.pixx.*;

/**
 * The AWT source.
 */
public class AWTSource extends Panel implements SourceListener,ActionListener,PixxScrollBarListener,FocusListener,StyledListListener,WindowListener,MouseWheelPanelListener,AWTStyleSelectorExListener
{
  protected Source _source;
  protected PixxVerticalScrollBar _scroll;
  protected Panel _panel;
  protected StyledList _list;
  protected AWTIrcTextField _textField;
  protected FormattedStringDrawer _styler;
  protected String _title;
  protected String _strippedTitle;
  private ListenerGroup _listeners;
  protected IRCConfiguration _ircConfiguration;
  protected AWTStyleSelectorEx _selector;
  protected boolean _topToBottom;
  private MouseWheelPanelWrapper _wrapper;

  /**
   * Create a new AWTSource.
   * @param config the global configuration.
   * @param source the source of this awt source.
   */
  public AWTSource(IRCConfiguration config,Source source)
  {
    this(config,source,false);
  }

  /**
   * Create a new AWTSource.
   * @param config the global configuration.
   * @param source the source of this awt source.
   * @param topToBottom true if the display should be performed from top to
   * bottom instead of bottom to top.
   */
  public AWTSource(IRCConfiguration config,Source source,boolean topToBottom)
  {
    _topToBottom=topToBottom;
    _ircConfiguration=config;
    _listeners=new ListenerGroup();
    _source=source;
    addFocusListener(this);
    _source.addSourceListener(this);
    _panel=new Panel();
    _panel.addFocusListener(this);
    _panel.setBackground(Color.white);
    _scroll=new PixxVerticalScrollBar(_ircConfiguration,0,0,0.1);
    _scroll.addPixxScrollBarListener(this);
    setLayout(new BorderLayout());

    Color aslMale=_ircConfiguration.getColor(PixxColorModel.COLOR_MALE);
    Color aslFemeale=_ircConfiguration.getColor(PixxColorModel.COLOR_FEMEALE);
    Color aslUndef=_ircConfiguration.getColor(PixxColorModel.COLOR_UNDEF);
    _list=new StyledList(_ircConfiguration,new Font(_ircConfiguration.getS("ChannelFontName"),Font.PLAIN,_ircConfiguration.getI("ChannelFontSize")),true,_ircConfiguration.getColorContext(source.getType(),source.getName()),aslMale,aslFemeale,aslUndef);
    if(_ircConfiguration.getB("RightToLeft")) _list.setHorizontalDirection(_list.RIGHT);
    _list.setBackgroundImage(_ircConfiguration.getBackgroundImage(source.getType(),source.getName()));
    _list.setBackgroundTiling(_ircConfiguration.getBackgroundTiling(source.getType(),source.getName()));

    _list.addFocusListener(this);
    _list.addStyledListListener(this);
    _styler=new FormattedStringDrawer(new Font("Monospaced",Font.PLAIN,12),_ircConfiguration,getColorContext());
    _textField=new AWTIrcTextField();

    Color[] c=_ircConfiguration.getStyleColors(getColorContext());
    _textField.setBackground(c[0]);
    _textField.setForeground(c[1]);
    _textField.addFocusListener(this);
    Panel p=new Panel();
    p.setLayout(new BorderLayout());

    _wrapper=new MouseWheelPanelWrapper(_panel);
    _wrapper.addMouseWheelPanelListener(this);
    p.add(_wrapper,"Center");

    //p.add(_panel,"Center");
    p.add(new PixxSeparator(PixxSeparator.BORDER_LEFT),"West");
    p.add(new PixxSeparator(PixxSeparator.BORDER_RIGHT),"East");
    p.add(new PixxSeparator(PixxSeparator.BORDER_UP),"North");
    p.add(new PixxSeparator(PixxSeparator.BORDER_DOWN),"South");

    add(p,"Center");
    _panel.setLayout(new BorderLayout());
    _panel.add(_scroll,"East");
    if(_ircConfiguration.getB("StyleSelector"))
    {
      p=new Panel(new BorderLayout());
      p.add(_textField,"Center");
      _selector=new AWTStyleSelectorEx(_ircConfiguration);
      _selector.addAWTStyleSelectorExListener(this);
      _selector.setColorContext(getColorContext());
      p.add(_selector,"East");
      add(p,"South");
    }
    else
    {
      _selector=null;
      add(_textField,"South");
    }

    if(_topToBottom)
      _list.setFirst(0);
    else
      _list.setLast(0);

    _textField.addActionListener(this);
    setTitle(_source.getName());
//    setSize(640,400);
    _panel.add(_list,"Center");
  }

  /**
   * Release this object. No further call may be performed on this object.
   */
  public void release()
  {
    _wrapper.removeMouseWheelPanelListener(this);
    _source.removeSourceListener(this);
    _panel.removeFocusListener(this);
    _scroll.removePixxScrollBarListener(this);
    _scroll.release();
    _list.removeFocusListener(this);
    _list.removeStyledListListener(this);
    _textField.removeFocusListener(this);
    if(_selector!=null)
    {
      _selector.removeAWTStyleSelectorExListener(this);
      _selector.release();
    }
    _textField.removeActionListener(this);
    _textField.release();

    _list.release();
    removeAll();
  }

  public Dimension getPreferredSize()
  {
    return new Dimension(600,300);
  }

  /**
   * Set the current textfield text.
   * @param text new textfield text.
   */
  public void setFieldText(String txt)
  {
    _textField.setText(txt);
    _textField.setCaretPosition(txt.length());
  }

  /**
   * Get the current textfield text.
   * @return the current textfield text.
   */
  public String getFieldText()
  {
    return _textField.getText();
  }

  /**
   * Validate the current textfield text, as if user pressed return key.
   */
  public void validateText()
  {
    _textField.validate();
  }

  /**
   * Get color context for this awt source.
   * @return color context for this awt source.
   */
	public ColorContext getColorContext()
	{
    return _ircConfiguration.getColorContext(_source.getType(),_source.getName());
	}

  /**
   * Get formatted text associated with the given text code, with no parameter.
   * @param code text code.
   * @return formatted text.
   */
  public String getText(int code)
  {
    return _ircConfiguration.getText(code);
  }

  /**
   * Get formatted text associated with the given text code, with one parameter.
   * @param code text code.
   * @param p1 first parameter.
   * @return formatted text.
   */
  public String getText(int code,String p1)
  {
    return _ircConfiguration.getText(code,p1);
  }

  /**
   * Get formatted text associated with the given text code, with two parameters.
   * @param code text code.
   * @param p1 first parameter.
   * @param p2 second parameter.
   * @return formatted text.
   */
  public String getText(int code,String p1,String p2)
  {
    return _ircConfiguration.getText(code,p1,p2);
  }

  /**
   * Get formatted text associated with the given text code, with three parameters.
   * @param code text code.
   * @param p1 first parameter.
   * @param p2 second parameter.
   * @param p3 third parameter.
   * @return formatted text.
   */
  public String getText(int code,String p1,String p2,String p3)
  {
    return _ircConfiguration.getText(code,p1,p2,p3);
  }

  /**
   * Add listener.
   * @param lis the listener to add.
   */
  public void addAWTSourceListener(AWTSourceListener lis)
  {
    _listeners.addListener(lis);
  }

  /**
   * Remove listener.
   * @param lis the listener to remove.
   */
  public void removeAWTSourceListener(AWTSourceListener lis)
  {
    _listeners.removeListener(lis);
  }

  /**
   * Set this source title.
   */
  public void setTitle(String title)
  {
	  if(title.equals(_title)) return;
    _title=title;
    _strippedTitle=_styler.getStripped(title);
    _listeners.sendEvent("titleChanged",this);
  }

  /**
   * Get the stripped title.
   * @return the stripped title.
   */
  public String getStrippedTitle()
  {
    return _strippedTitle;
  }

  /**
   * Get the title.
   * @return the title.
   */
  public String getTitle()
  {
    return _title;
  }

  /**
   * Get a shorter title.
   * @return a shorter title.
   */
  public String getShortTitle()
  {
    return _source.getName();
  }

  /**
   * Get the source.
   * @return the source.
   */
  public Source getSource()
  {
    return _source;
  }

  public void actionPerformed(ActionEvent e)
  {
    if(_textField.getText().length()==0) return;
    String prefix="";
    if(_selector!=null) prefix=_selector.getPrefix();

    if(!_textField.getText().startsWith("/"))
    {
      _source.sendString(prefix+_textField.getText());
    }
    else
    {
      String txt=_textField.getText();
      String cmd=txt;
      int pos=cmd.indexOf(' ');
      if(pos!=-1) cmd=cmd.substring(0,pos);
      if(_ircConfiguration.mayCommand(cmd.substring(1))) _source.sendString(txt);
    }
    if(_textField!=null) _textField.setText("");

  }

  public void fontSelected(Font fnt)
  {
    _list.setFont(fnt);
  }


  /**
   * Clear this awt source display.
   */
  public void clear()
  {
    _list.clear();
    _scroll.setMaximum(_list.getLineCount()-1);
    _scroll.setValue(_list.getLast());
    _listeners.sendEvent("eventOccured",this);
  }

  /**
   * Test wether the specified line should be highlighted.
   * @param msg line to test.
   * @return true if msg should be highlighted, false otherwise.
   */
  protected boolean needHighLight(String msg)
  {
    msg=msg.toLowerCase();
    if(_ircConfiguration.highLightNick())
    {
      String myNick=_source.getServer().getNick().toLowerCase();
      if(msg.indexOf(myNick)!=-1) return true;
    }

    Enumeration e=_ircConfiguration.getHighLightWords();
    while(e.hasMoreElements())
    {
      String word=((String)(e.nextElement())).toLowerCase();
      if(msg.indexOf(word)!=-1) return true;
    }
    return false;
  }

  /**
   * Test wether the specified line should trigger "on word" sounds. Play the
   * sounds associated with the words.
   * @param msg line to test.
   */
  protected void checkSound(String msg)
  {
    msg=msg.toLowerCase();
    AudioConfiguration ac=_ircConfiguration.getAudioConfiguration();
    Enumeration e=ac.getSoundWords();
    while(e.hasMoreElements())
    {
      String word=((String)(e.nextElement())).toLowerCase();
      if(msg.indexOf(word)!=-1) ac.onWord(word);
    }
  }

  /**
   * Print the given message on the awt source display.
   * @param mes message to print.
   * @param color color to use.
   * @param bold true if message should be in bold.
   * @param underline if message should be underlined.
   */
  protected void print(String msg,int color,boolean bold,boolean underline)
  {
    _source.activate();
    if(color!=1) msg="\3"+color+msg;
    if(bold) msg="\2"+msg;
    if(underline) msg=((char)31)+msg;



    if(_ircConfiguration.getB("TimeStamp"))
    {
      Calendar cal=Calendar.getInstance();
      String hour=""+cal.get(Calendar.HOUR_OF_DAY);
      if(hour.length()==1) hour="0"+hour;
      String min=""+cal.get(Calendar.MINUTE);
      if(min.length()==1) min="0"+min;
      msg="["+hour+":"+min+"] "+msg;
    }
    _list.addLine(msg);
    _scroll.setMaximum(_list.getLineCount()-1);
    _scroll.setValue(_list.getLast());
    _listeners.sendEvent("eventOccured",this);
  }

  /**
   * Print the given message on the awt source display.
   * @param mes message to print.
   * @param color color to use.
   */
  protected void print(String msg,int color)
  {
    print(msg,color,false,false);
  }

  /**
   * Print the given message on the awt source display.
   * @param msg message to print.
   */
  protected void print(String msg)
  {
    print(msg,1,false,false);
  }

  public void messageReceived(String source,String str)
  {
    checkSound(str);
    if(needHighLight(str))
    {
      print("<"+source+"> "+str,_ircConfiguration.getI("HighLightColor"));
    }
    else
    {
      print("<"+source+"> "+str);
    }
  }

  public void reportReceived(String msg)
  {
    print(msg);
  }

  public void noticeReceived(String from,String msg)
  {
    print("-"+from+"- "+msg,5);
  }

  public void action(String nick,String msg)
  {
    print("* "+nick+" "+msg,6);
  }

  public void activate()
  {
    _listeners.sendEvent("activated",this);
  }

  /**
   * Return the active state for this awt source.
   * @return awt source active state.
   */
  public boolean isActive()
  {
    return _source.isActive();
  }

  /**
   * Leave this awt source.
   */
  public void leave()
  {
    _source.leave();
  }

  public void mouseWheelMoved(Integer amount)
  {
    int i=amount.intValue();
    _scroll.setValue(_scroll.getValue()+i);
    if(_topToBottom)
      _list.setFirst(_scroll.getValue());
    else
      _list.setLast(_scroll.getValue());
  }

  public void valueChanged(PixxScrollBar pixScrollBar)
  {
    if(_topToBottom)
      _list.setFirst(_scroll.getValue());
    else
      _list.setLast(_scroll.getValue());
  }

  public void focusGained(FocusEvent e)
  {
    if(e.getComponent()!=_textField)
      _textField.requestFocus();
  }

  public void focusLost(FocusEvent e)
  {
  }

  public void channelEvent(StyledList lis,String chan,MouseEvent e)
  {
    if(_ircConfiguration.matchMouseConfiguration("ChannelJoin",e))
    {
      _source.sendString("/join "+chan);
    }
  }

  public void URLEvent(StyledList lis,String url,MouseEvent e)
  {
    if(_ircConfiguration.matchMouseConfiguration("URLOpen",e))
    {
      _source.sendString("/url "+url);
    }
  }

  public void nickEvent(StyledList lis,String nick,MouseEvent e)
  {
    if(_ircConfiguration.matchMouseConfiguration("NickQuery",e))
    {
      if(nick.equals(getSource().getServer().getNick())) return;
      if(_ircConfiguration.getB("AutomaticQueries"))
        _source.sendString("/query "+nick);
    }
  }

  public String toString()
  {
    return "AWTSource : "+getStrippedTitle();
  }

  public void setVisible(boolean b)
  {
    super.setVisible(b);
    if(!b) _list.dispose();
  }

  public void copyEvent(StyledList lis,String txt,MouseEvent e)
  {
    Frame f=new Frame();
    f.setTitle(getText(TextProvider.GUI_COPY_WINDOW));
    f.addWindowListener(this);
    f.setLayout(new GridLayout(1,1));
    Panel p=new Panel();
    p.setLayout(new GridLayout(1,1));
    f.add(p);
    TextArea c=new TextArea();
    c.setFont(new Font("",Font.PLAIN,12));

    c.setText(txt);
    p.add(c);
    f.setSize(400,300);
    f.show();
  }

  public void virtualSizeChanged(StyledList lis)
  {
  }

  public void windowActivated(WindowEvent e) {}
  public void windowClosed(WindowEvent e) {}
  public void windowClosing(WindowEvent e)
  {
    e.getWindow().hide();
    e.getWindow().dispose();
  }
  public void windowDeactivated(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowIconified(WindowEvent e) {}
  public void windowOpened(WindowEvent e) {}

}

