/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc.gui.pixx;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import irc.gui.*;
import irc.*;

class MDILayout implements LayoutManager
{
  private Hashtable _components;

  public MDILayout()
  {
    _components=new Hashtable();
  }

  public void addLayoutComponent(String name, Component comp)
  {
    _components.put(comp,comp);
  }

  private Component getVisible(Container parent)
  {
    Component[] c=parent.getComponents();
    for(int i=0;i<c.length;i++) if(c[i].isVisible()) return c[i];
    return null;
  }

  public void layoutContainer(Container parent)
  {
    Component c=getVisible(parent);
    if(c==null) return;
    int w=parent.getSize().width;
    int h=parent.getSize().height;
    c.setBounds(0,0,w,h);
  }

  public Dimension minimumLayoutSize(Container parent)
  {
    return new Dimension(0,0);
  }

  public Dimension preferredLayoutSize(Container parent)
  {
    Component visible=getVisible(parent);
    if(visible!=null) return visible.getPreferredSize();
    return new Dimension(0,0);
  }

  public void removeLayoutComponent(Component comp)
  {
    _components.remove(comp);
  }
}

/**
 * The multiplie document interface.
 */
public class PixxMDIInterface extends PixxPanel implements PixxTaskBarListener,PixxMenuBarListener,ActionListener,MouseWheelPanelListener
{
  private PixxMenuBar _menu;
  private PixxTaskBar _task;
  private Panel _mdi;
  private PopupMenu _popMenu;
  private ListenerGroup _listeners;
	private TextField _nickField;
	private Hashtable _awt2Dock;
	private AWTInterpretor _interpretor;

  /**
   * Create a new PixxMDIInterface.
   * @param config global irc configuration.
   * @param task taskbar.
   * @param title true if this mdi should display its title.
   */
  public PixxMDIInterface(IRCConfiguration config,PixxTaskBar task,boolean title)
  {
    super(config);
    _interpretor=new AWTInterpretor(config,this);
    _awt2Dock=new Hashtable();
    _popMenu=new PopupMenu();
    _listeners=new ListenerGroup();
    setLayout(new BorderLayout());
    _mdi=new Panel();
    _mdi.setLayout(new MDILayout());
    _mdi.setBackground(Color.white);
    _task=task;
    _task.add(_popMenu);
    _popMenu.addActionListener(this);
    _task.addPixxTaskBarListener(this);
    _menu=new PixxMenuBar(_ircConfiguration,title);
    _menu.addPixxMenuBarListener(this);
    add(_menu,"North");
    add(_mdi,"Center");
		_nickField=new NonFocusableTextField("");
		_nickField.addActionListener(this);
		if(!_ircConfiguration.getB("NickField"))
		{
      add(_task,"South");
		}
		else
		{
		  Panel bottom=new Panel();
			bottom.setLayout(new BorderLayout());
			bottom.add(_task,"Center");
			Panel nickConfig=new Panel();
			nickConfig.setLayout(new BorderLayout());
			Label label=new Label(getText(TextProvider.GUI_CHANGE_NICK));
		  label.setBackground(_ircConfiguration.getColor(PixxColorModel.COLOR_BACK));
		  label.setForeground(_ircConfiguration.getColor(PixxColorModel.COLOR_WHITE));
			Panel outerNickLabel=new Panel();
			outerNickLabel.setLayout(new BorderLayout());
			outerNickLabel.add(label,"Center");
      outerNickLabel.add(new PixxSeparator(PixxSeparator.BORDER_LEFT),"West");
      outerNickLabel.add(new PixxSeparator(PixxSeparator.BORDER_RIGHT),"East");
      outerNickLabel.add(new PixxSeparator(PixxSeparator.BORDER_UP),"North");
      outerNickLabel.add(new PixxSeparator(PixxSeparator.BORDER_DOWN),"South");

			nickConfig.add(outerNickLabel,"North");
			nickConfig.add(_nickField,"Center");

      bottom.add(nickConfig,("East"));
			add(bottom,"South");
		}
    validate();
  }

  public void release()
  {
    _popMenu.removeActionListener(this);
    _task.removePixxTaskBarListener(this);
    _menu.removePixxMenuBarListener(this);
    _nickField.removeActionListener(this);
    _task=null;
    _menu=null;
    _popMenu=null;
    _awt2Dock=null;
    super.release();
  }

  /**
   * Get the taskbar associated with this interface.
   * @return the taskbar.
   */
  public PixxTaskBar getTaskBar()
  {
    return _task;
  }

  /**
   * Set the title.
   * @param title title string.
   * @param context title color context.
   */
  public void setTitle(String title,ColorContext context)
  {
    _menu.setTitle(title,context);
  }

  /**
   * Add a listener.
   * @param lis listener to add.
   */
  public void addPixxMDIInterfaceListener(PixxMDIInterfaceListener lis)
  {
    _listeners.addListener(lis);
  }

  /**
   * Remove a listener.
   * @param lis listener to remove.
   */
  public void removePixxMDIInterfaceListener(PixxMDIInterfaceListener lis)
  {
    _listeners.removeListener(lis);
  }

  /**
   * Set the connected state.
   * @param b true if connected to server, false otherwise.
   */
  public void setConnected(boolean b)
  {
    _menu.setConnected(b);
  }

  private void test()
  {
    //if(_task.getActive()==_lastActive) return;
    //_lastActive=_task.getActive();
    //triggerChangedListeners(_lastActive);
    _listeners.sendEvent("activeChanged",_task.getActive(),this);
  }

  /**
   * Get active awt source.
   * @return the current active awt source.
   */
  public AWTSource getActive()
  {
    return _task.getActive();
  }

  private DockablePanel createSource(AWTSource asource)
  {
    DockablePanel source=new DockablePanel(asource,getColor(COLOR_BACK));
    _awt2Dock.put(asource,source);
    return source;
  }

  private DockablePanel getSource(AWTSource source)
  {
    return (DockablePanel)_awt2Dock.get(source);
  }

  private void deleteSource(AWTSource source)
  {
    _awt2Dock.remove(source);
  }

  /**
   * Dock the specified source.
   * @param source the source to be docked.
   */
  public void dock(AWTSource source)
  {
    getSource(source).dock();
    source.requestFocus();
  }

  /**
   * Undock the specified source.
   * @param source the source to be undocked.
   */
  public void undock(AWTSource source)
  {
    getSource(source).undock(source.getShortTitle());
    source.requestFocus();
  }

  /**
   * Check whether the given source is docked or not.
   * @param source the source to be checked.
   * @return true if the source is docked, false otherwise.
   */
  public boolean isDocked(AWTSource source)
  {
    return getSource(source).isDocked();
  }

  /**
   * Get the interpretor for this interface. Any unrecognized command will be passed
   * to this interpretor.
   * @return the interpretor for this interface, or null if no default interface
   * interpretor should be used.
   */
  public Interpretor getInterpretor()
  {
    return _interpretor;
  }

  /**
   * Find the AWTSource relative to the given source.
   * @param src source to find AWTSource from.
   * @return AWTSource relative to the given source, or null if not found.
   */
  public AWTSource findAWTSource(Source src)
  {
    Enumeration e=_awt2Dock.keys();
    while(e.hasMoreElements())
    {
      AWTSource asrc=(AWTSource)e.nextElement();
      if(asrc.getSource()==src) return asrc;
    }
    return null;
  }

  public void AWTSourceAdded(PixxTaskBar bar,AWTSource asource)
  {
    DockablePanel source=createSource(asource);
    _mdi.add(source);
    source.setVisible(false);
    validate();
    test();
    if(_ircConfiguration.getDockingPolicy(asource.getSource().getType().toLowerCase(),asource.getSource().getName().toLowerCase())) undock(asource);
  }

  public void AWTSourceRemoved(PixxTaskBar bar,AWTSource asource)
  {
    DockablePanel source=getSource(asource);
    source.dock();
    deleteSource(asource);
    _mdi.remove(source);
    source.release();
    validate();
    test();
  }

  public void AWTSourceDesactivated(PixxTaskBar bar,AWTSource asource)
  {
    DockablePanel source=getSource(asource);
    source.setVisible(false);
    validate();
    test();
  }

  public void AWTSourceActivated(PixxTaskBar bar,AWTSource asource)
  {
    if(asource!=null)
    {
      DockablePanel source=getSource(asource);
      source.setVisible(true);
      validate();
      source.requestFocus();
    }
    test();
  }

  public void eventOccured(PixxTaskBar bar,AWTSource asource,MouseEvent e)
  {
    DockablePanel source=getSource(asource);
    if(_ircConfiguration.matchMouseConfiguration("TaskBarPopup",e))
    {
      //_selectedSource=asource;

      _popMenu.removeAll();
      _popMenu.add(new MenuItem(getText(TextProvider.GUI_CLOSE)));

      _popMenu.show(_task,e.getX(),e.getY());
    }
    source.requestFocus();
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getActionCommand().equals(getText(TextProvider.GUI_CLOSE)))
    {
      //_selectedSource.leave();

      AWTSource src=_task.getActive();
      if(src!=null) src.leave();
    }
		else if(e.getSource()==_nickField)
		{
      AWTSource src=_task.getActive();
      if(src==null) return;
		  src.getSource().sendString("/nick "+_nickField.getText());
		}
  }

  public void connectionClicked(PixxMenuBar bar)
  {
    _listeners.sendEvent("connectTriggered",this);
  }

  public void chanListClicked(PixxMenuBar bar)
  {
    AWTSource src=_task.getActive();
    if(src==null) return;
    src.getSource().sendString("/list");
  }

  public void aboutClicked(PixxMenuBar bar)
  {
    _listeners.sendEvent("aboutTriggered",this);
  }

  public void helpClicked(PixxMenuBar bar)
  {
    _listeners.sendEvent("helpTriggered",this);
  }

  public void closeClicked(PixxMenuBar bar)
  {
    AWTSource src=_task.getActive();
    if(src==null) return;
    src.leave();
  }

  public void dockClicked(PixxMenuBar bar)
  {
    AWTSource src=_task.getActive();
    if(src==null) return;
    if(isDocked(src))
      undock(src);
    else
      dock(src);
  }

  public void mouseWheelMoved(Integer amount)
  {
    AWTSource src=_task.getActive();
    if(src==null) return;
    src.mouseWheelMoved(amount);
  }

}

